@@ -1,5 +1,5 @@
 # Locale::Po4a::Pod -- Convert POD data to PO file, for translation.
-# $Id: Chooser.pm,v 1.28 2005/11/17 23:44:33 nekral-guest Exp $
+# $Id: Chooser.pm,v 1.32 2006/06/12 18:33:30 nekral-guest Exp $
 #
 # This program is free software; you may redistribute it and/or modify it
 # under the terms of GPL (see COPYING).
@@ -34,6 +34,8 @@ sub new {
         $modname = 'NewsDebian';
     } elsif ($module eq 'latex') {
         $modname = 'LaTeX';
+    } elsif ($module eq 'texinfo') {
+        $modname = 'Texinfo';
     } elsif ($module eq 'text') {
         $modname = 'Text';
     } else {
@@ -64,6 +66,7 @@ sub list {
 	."\n  - ".gettext("guide: Gentoo Linux's xml documentation format.")
 	."\n  - ".gettext("docbook: Docbook XML.")
 	."\n  - ".gettext("latex: LaTeX format.")
+	."\n  - ".gettext("texinfo: The info page format.")
     );
     exit shift;
 }
@@ -103,13 +106,16 @@ L<Locale::Po4a::Po(3pm)>
 
 =item About modules:
 
+L<Locale::Po4a::Dia(3pm)>,
+L<Locale::Po4a::Html(3pm)>,
 L<Locale::Po4a::KernelHelp(3pm)>,
+L<Locale::Po4a::LaTeX(3pm)>,
 L<Locale::Po4a::Man(3pm)>,
 L<Locale::Po4a::Pod(3pm)>,
 L<Locale::Po4a::Sgml(3pm)>,
-L<Locale::Po4a::Html(3pm)>,
-L<Locale::Po4a::Dia(3pm)>,
-L<Locale::Po4a::LateX(3pm)>.
+L<Locale::Po4a::Texinfo(3pm)>,
+L<Locale::Po4a::Text(3pm)>,
+L<Locale::Po4a::Xml(3pm)>.
 
 =back
 
