@@ -7,7 +7,7 @@
 # This code extracts plain text from tags and attributes on Docbook XML
 # documents.
 #
-# Copyright (c) 2004 by Jordi Vilalta  <jvprat@wanadoo.es>
+# Copyright (c) 2004 by Jordi Vilalta  <jvprat@gmail.com>
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -58,11 +58,11 @@ L<po4a(7)|po4a.7>, L<Locale::Po4a::TransTractor(3pm)>, L<Locale::Po4a::Xml(3pm)>
 
 =head1 AUTHORS
 
- Jordi Vilalta <jvprat@wanadoo.es>
+ Jordi Vilalta <jvprat@gmail.com>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2004 by Jordi Vilalta  <jvprat@wanadoo.es>
+Copyright (c) 2004 by Jordi Vilalta  <jvprat@gmail.com>
 
 This program is free software; you may redistribute it and/or modify it
 under the terms of GPL (see the COPYING file).
@@ -88,12 +88,16 @@ sub initialize {
 	$self->{options}{'tagsonly'}=1;
 	$self->{options}{'wrap'}=1;
 	$self->{options}{'doctype'}=$self->{options}{'doctype'} || 'docbook xml';
-	$self->{options}{'tags'}.='
+	my ($additional_tags, $additional_inline) = ("", "");
+	my $tag;
+	foreach $tag qw(
 		<abbrev>
 		<acronym>
 		<arg>
 		<artheader>
 		<attribution>
+		<citetitle>
+		<computeroutput>
 		<date>
 		<entry>
 		<figure>
@@ -112,15 +116,18 @@ sub initialize {
 		<pubdate>
 		<publishername>
 		<primary>
+		<prompt>
+		<quote>
 		<refclass>
 		<refdescriptor>
-		<refentrytitle>
 		<refmiscinfo>
 		<refname>
 		<refpurpose>
 		<releaseinfo>
 		<remark>
+		<replaceable>
 		<revnumber>
+		W<screen>
 		<screeninfo>
 		<seg>
 		<segtitle>
@@ -129,12 +136,18 @@ sub initialize {
 		<synopfragmentref>
 		<term>
 		<title>
-		<titleabbrev>';
-	$self->{options}{'inline'}.='
+		<titleabbrev>
+		<userinput>) {
+		if (not defined $self->{nodefault}{$tag}) {
+			$additional_tags .= " $tag";
+		}
+	}
+	foreach $tag qw(
 		<action>
 		<affiliation>
 		<anchor>
 		<application>
+		<arg>
 		<author>
 		<authorinitials>
 		<command>
@@ -205,14 +218,15 @@ sub initialize {
 		<prompt>
 		<property>
 		<quote>
-		<remark>
+		<refentrytitle>
 		<replaceable>
+		<remark>
 		<returnvalue>
 		<revhistory>
 		<sgmltag>
 		<sidebar>
 		<structfield>
-		<structname
+		<structname>
 		<subscript>
 		<superscript>
 		<surname>
@@ -226,7 +240,13 @@ sub initialize {
 		<varname>
 		<wordasword>
 		<xref>
-		<year>';
+		<year>) {
+		if (not defined $self->{nodefault}{$tag}) {
+			$additional_inline .= " $tag";
+		}
+	}
+	$self->{options}{'tags'} .= $additional_tags;
+	$self->{options}{'inline'} .= $additional_inline;
 	$self->{options}{'attributes'}.='
 		lang';
 	$self->treat_options;
