@@ -85,7 +85,7 @@ translations.
 =item non-breaking spaces
 
 Translators can use non-breaking spaces in their translations.  These
-non-breaking spaces (0xA0) will be transliterated into a roff
+non-breaking spaces (0xA0 in latin1) will be transliterated into a roff
 non-breaking space ('\ ').
 
 =item quotes transliterations
@@ -195,6 +195,13 @@ not split the current paragraph. The string to translate will then contain
 I<foo E<lt>.bar baz quxE<gt> quux>, where I<foo> is the command that
 should be inlined.
 
+=item generated
+
+This option specifies that the file was generated, and that po4a should not
+try to detect if the man pages was generated from another format.
+This permits to use po4a on generated man pages.
+This option does not take any argument.
+
 =back
 
 =head1 AUTHORING MAN PAGES COMPLIANT WITH PO4A::MAN
@@ -212,14 +219,6 @@ not really inclined to correct, and which will constitute some pitfalls
 you'll have to avoid if you want to see translators taking care of your
 documentation.
 
-=head2 Don't use the mdoc macro set
-
-The macro set described in mdoc(7) (and widely used under BSD, IIRC) isn't
-supported at all by po4a, and won't be. It would need a completely separate
-parser for this, and I'm not inclined to do so. On my machine, there are
-only 63 pages based on mdoc, from 4323 pages. If someone implements the mdoc
-support, I'll happily include this, though.
-
 =head2 Don't program in nroff
 
 nroff is a complete programming language, with macro definition,
@@ -227,13 +226,6 @@ conditionals and so on. Since this parser isn't a fully featured nroff
 interpreter, it will fail on pages using these facilities (There are about
 200 such pages on my box).
 
-=head2 Avoid file inclusion when possible
-
-The '.so' groff macro used to include another file in the current one is
-supported, but from my own experience, it makes harder to manipulate
-the man page, since all files have to be installed in the right location so
-that you can see the result (ie, it breaks somehow the '-l' option of man).
-
 =head2 Use the plain macro set
 
 There are still some macros which are not supported by po4a::man. This is
@@ -285,48 +277,34 @@ with a fix when possible...
 
 =head1 STATUS OF THIS MODULE
 
-I think that this module is still beta, but could be used for most of the
-existing man pages. I ran some test, processing all pages of my box and
-diff'ing between the original and the version processed trough po4a. The
-results are the following:
+This module can be used for most of the existing man pages.
 
- # of pages         : 5060
+Some tests are regularly run on Linux boxes:
 
- Ignored pages      : 1742 (34%)
- parser fails       :  530 (12% of all; 18% of unignored)
+=over 4
 
- works perfectly    : 1947 (39% of all; 59% of unignored; 70% of processed)
- change wrapping    :  409 ( 8% of all; 12% of unignored; 15% of processed)
- change wrapping
- and/or font        :  364 ( 7% of all; 11% of unignored; 13% of processed)
+=item *
 
- undetected problems:   68 ( 1% of all;  2% of unignored;  2% of processed)
+one third of the pages are refused because they were generated from
+another format suported by po4a (e.g. pod or SGML).
 
-Ignored pages are so, because they are not the source files. They are for
-example generated from POD or SGML. In that case, you should translate the
-[real] source file with the relevant po4a module instead of the generated
-man page.
+=item *
 
-Parser fails on pages based on mdoc(7), pages using conditionals with .if,
-defining new macros with .de, using non standard fonts, and more generally,
-not following the advices of the previous section.
+10% of the remaining pages are rejected with an error (e.g. a
+groff macro is not supported).
 
-Pages with undetected problems are processed without complain by po4a::man,
-but the generated output is different from the original one (some strings
-are present in the original page and not in the one normalized by po4a, or
-the contrary). All of them are bugs, but most of the time this exhibit
-issues in the original page.
+=item *
 
-Most of the pages in the "change wrapping and/or font" category will only
-have their wrapping changed (but it was too difficult to figure this out
-automatically), or have other more serious formatting change (ie, which
-chars are italics, which ones are bold, etc.).
+Then, less than 1% of the pages are accepted silently by po4a, but with
+significant issues (i.e. missing words, or new words inserted)
 
-So, it seems like since ignored pages are translatable with po4a::pod and
-since wrapping changes are acceptable in most cases, the current version
-of po4a can translate 80% of the man pages on my machine. Moreover, most of
-the untranslatable pages could be fixed with some simple tricks given
-above. Isn't that coooool?
+=item *
+
+The other pages are usually handled without differences more important
+than spacing differences or line rewrapped (font issues in less than 10% of
+the processed pages).
+
+=back
 
 =head1 SEE ALSO
 
@@ -341,7 +319,7 @@ L<Locale::Po4a::Pod(3pm)>.
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 2002, 2003, 2004, 2005 by SPI, inc.
+Copyright 2002, 2003, 2004, 2005, 2006 by SPI, inc.
 
 This program is free software; you may redistribute it and/or modify it
 under the terms of GPL (see the COPYING file).
@@ -378,6 +356,9 @@ my $FONT_RE = "\\\\f(?:\\[[^\\]]*\\]|\\(..|[^\\(\\[])";
 # into the groff non-breaking space.
 my $nbs;
 
+# Indicate if the page uses the mdoc macros
+my $mdoc_mode = 0;
+
 #########################
 #### DEBUGGING STUFF ####
 #########################
@@ -408,6 +389,9 @@ my %no_wrap_end = (
 );
 my %inline = (
 );
+# This variable indicates whether po4a should try to detect the generated
+# files.
+my $allow_generated = 0;
 sub initialize {
     my $self = shift;
     my %options = @_;
@@ -421,6 +405,7 @@ sub initialize {
     $self->{options}{'translate_each'}='';
     $self->{options}{'no_wrap'}='';
     $self->{options}{'inline'}='';
+    $self->{options}{'generated'}='';
 
     # reset the debug options
     %debug = ();
@@ -483,6 +468,9 @@ sub initialize {
             $inline{$_} = 1;
         }
     }
+    if (defined $options{'generated'}) {
+        $allow_generated = 1;
+    }
 }
 
 my @comments = ();
@@ -527,9 +515,10 @@ NEW_LINE:
     $line =~ s/\\\././g;
 
     chomp $line;
-    if ($line =~ m/^(.*?)(?:(?<!\\)\\"(.*))$/) {
-        my ($l, $c) = ($1, $2);
+    if ($line =~ m/^(.*?)(?:(?<!\\)\\(["#])(.*))$/) {
+        my ($l, $t, $c) = ($1, $2, $3);
         $line = $l;
+        unless ($allow_generated) {
         # Check for comments indicating that the file was generated.
         if ($c =~ /Pod::Man/) {
             warn wrap_mod("po4a::man", dgettext("po4a", "This file was generated with Pod::Man. Translate the pod file with the pod module of po4a."));
@@ -562,6 +551,7 @@ NEW_LINE:
                 "You should translate the source file, but continuing anyway."
                 ),$l."\\\"".$c);
         }
+        }
 
         if ($line =~ m/^[.']*$/) {
             if ($c !~ m/^\s*$/) {
@@ -573,6 +563,10 @@ NEW_LINE:
                 # (empty lines are a little bit different)
                 goto NEW_LINE;
             }
+            if ($line =~ m/^\s*$/ and $t eq "#") {
+                # Groff comments
+                goto NEW_LINE;
+            }
         } else {
             push @comments, $c;
         }
@@ -582,22 +576,36 @@ NEW_LINE:
         push @comments, @next_comments;
         @next_comments = ();
     }
+
+    # A .I or .B request chnage the current font
+    # and on exit, switch the font to Roman
+    # When one of these request doesn't have its argument on its line
+    # (and when we support this usage), we must keep this font request to
+    # insert it later.
+    # It is a stack of fonts to be inserted (in case a .I is followed by
+    # a .B and then followed bysome text; note that in this case,
+    # only one \fR must be inserted at the end of the text)
+    my $insert_font = "";
     while ($line =~ /\\$/ || $line =~ /^(\.[BI])\s*$/) {
         my ($l2,$r2)=$self->SUPER::shiftline();
         chomp($l2);
         if ($line =~ /^(\.[BI])\s*$/) {
-            if (   $l2 =~ /^[.'][\t ]*([BIR]|BI|BR|IB|IR|RB|RI|SH|TP)[\t ]/
-                or $l2 =~ /^[.'][\t ]*([BIR])$/) {
-                # another font macro. Forget about the first one
+            if ($l2 =~ /^[.'][\t ]*([BI]|BI|BR|IB|IR|RB|RI)(?:[\t ]|\s*$)/) {
+                my $font = $line;
+                $font =~ s/^\.([BI])\s*$/$1/;
+                $insert_font = "\\f$font$insert_font";
                 $line = $l2;
                 $ref = $r2;
+            } elsif ($l2 =~ /^[.'][\t ]*(SH|TP|P|PP|LP)(?:[\t ]|\s*$)/) {
+                $line =~ s/^\.([BI])\s*$/$insert_font\\f$1/;
+                $self->SUPER::unshiftline($l2,$r2);
             } elsif ($l2 =~ /^([.'][\t ]*(?:IP)[\t ]+"?)(.*)$/) {
                 # Install the font modifier into the next line
                 # after a possible quote (")
                 my $macro = $1;
                 my $arg   = $2;
                 $line =~ /^\.([BI])\s*$/;
-                $line = $macro."\\f$1".$arg;
+                $line = $macro."$insert_font\\f$1".$arg;
                 $ref = $r2;
             } elsif ($l2 =~ /^[.']/) {
                 die wrap_ref_mod($ref, "po4a::man", dgettext("po4a",
@@ -621,18 +629,23 @@ NEW_LINE:
             $line .= $l2;
         }
     }
+    # Detect non-wrapped paragraphs
+    # This must be done before handling the .B, .RI ... font requests
+    $line =~ s/^($FONT_RE)(\s+)/$2$1/;
+
     $line .= "\n";
 
     # Handle font requests here
-    if ($line =~ /^[.'][\t ]*([BIR]|BI|BR|IB|IR|RB|RI)(?:(?: +|\t)(.*)|)$/) {
+    if ($line =~ /^[.'][\t ]*([BI]|BI|BR|IB|IR|RB|RI)(?:(?: +|\t)(.*)|)$/) {
         my $macro = $1;
         my $arguments = $2;
         my @args = splitargs($ref,$arguments);
-        if ($macro eq 'B' || $macro eq 'I' || $macro eq 'R') {
+        if ($macro eq 'B' || $macro eq 'I') {
             my $arg=join(" ",@args);
             $arg =~ s/^ +//;
             this_macro_needs_args($macro,$ref,$arg);
-            $line = "\\f$macro".$arg."\\fR\n";
+            $line = "$insert_font\\f$macro".$arg."\\fR\n";
+            $insert_font = "";
         }
         # .BI bold alternating with italic
         # .BR bold/roman
@@ -648,6 +661,28 @@ NEW_LINE:
                                     "\\f$b$_" :
                                     "\\f$a$_"
                                  } @args)."\\fR\n";
+            if ($i eq 0) {
+                # If a .BI is used without argument, we must insert a
+                # \fI\fR. The \fR was inserted previously.
+                $line = "\\f$b$line";
+            }
+        }
+
+        if (length $insert_font) {
+            $line =~ s/\n$//;
+            $line = "$insert_font$line\\fR\n";
+        }
+
+        if ($line =~ /^(.*)\\c(\\f.)?\s*\\fR\n/) {
+            my $begin = $1;
+
+            my ($l2,$r2)=$self->SUPER::shiftline();
+            if ($l2 =~ /^[.']/) {
+                $self->SUPER::unshiftline($l2,$r2);
+            } else {
+                $l2 =~ s/\s*$//s;
+                $line = "$begin\\fR$l2\n";
+            }
         }
     }
 
@@ -740,6 +775,12 @@ sub pre_trans {
     }
 
     # Note: if you want to implement \c support, the gdb man page is your playground
+    if (    not defined $self->{type}) {
+        $str =~ s/(\G|^(?:.*?)\n|^)        # Last position, or begin of a line
+                   ([ \t]*[^.'][^\n]*(?<!\\)(?:\\\\)*) # the new line, which
+                   \\c[ \t]*\n             # ends by \c and followed by a line
+                   (?![ \t]*[.'])/$1$2/sgx;# not followed by a command (.')
+    }
     die wrap_ref_mod($ref, "po4a::man", dgettext("po4a","Escape sequence \\c encountered. This is not completely handled yet."))
 	if ($str =~ /\\c/);
 
@@ -750,6 +791,17 @@ sub pre_trans {
     while ($str =~ m/^(.*)PO4A-INLINE:(.*?):PO4A-INLINE(.*)$/s) {
         my ($t1,$t2, $t3) = ($1, $2, $3);
         $str = "$1E<$2>";
+        if ($mdoc_mode) {
+            # When a punctuation sign must be joined to an argument, mdoc
+            # permits to use such a construct:
+            # .Ar file1 , file2 , file3 ) .
+            # Here, we move the punctuation out of the E<...> tag.
+            # This is reverted in post_trans.
+            # FIXME: To be checked with the French punctuation
+            while ($str =~ m/ +([.,;:\)\]]) *>/s) {
+                $str =~ s/ +([.,;:\)\]]) *>/>$1/s;
+            }
+        }
         if (defined $t3 and length $t3) {
             $t3 =~ s/^\n//s;
             $str .= "\n$t3";
@@ -760,7 +812,7 @@ sub pre_trans {
     if (defined $self->{type} && $self->{type} =~ m/^(SH|SS)$/) {
         set_regular("B");
     }
-    $str = do_fonts($str);
+    $str = do_fonts($str, $ref);
     if (defined $self->{type} && $self->{type} =~ m/^(SH|SS)$/) {
         set_regular("R");
     }
@@ -774,6 +826,7 @@ sub pre_trans {
         $str = $2;
     }
 
+    unless ($mdoc_mode) {
     # Kill minus sign/hyphen difference.
     # Aestetic of printed man pages may suffer, but:
     #  * they are translator-unfriendly
@@ -786,6 +839,7 @@ sub pre_trans {
     $str =~ s/\\\*\(lq/``/sg;
     $str =~ s/\\\*\(rq/''/sg;
     $str =~ s/\\\(dq/"/sg;
+    }
     
     # non-breaking spaces
     # some non-breaking spaces may have been added during the parsing
@@ -807,6 +861,7 @@ sub post_trans {
         return $str;
     }
 
+    unless ($mdoc_mode) {
     # Post formatting, so that groff see the strange chars
     $str =~ s|\\-|-|sg; # in case the translator added some of them manually
     # change hyphens to minus signs
@@ -818,13 +873,17 @@ sub post_trans {
             my $begin = $1;
             $str = $2;
             my $tmp2 = $tmp.$begin;
-            if (   ($begin =~ m/\\s$/s)
-                or ($begin =~ m/\\\((.|E<[gl]t>)$/s)
-                or ($tmp2 =~ m/\\h'([^']|\\')*$/)) {
+            if (   ($begin =~ m/(?<!\\)(\\\\)*\\s$/s)
+                or ($begin =~ m/(?<!\\)(\\\\)*\\\((.|E<[gl]t>)$/s)
+                or ($tmp2 =~ m/(?<!\\)(\\\\)*\\[hCv]'([^']|(?<!\\)(\\\\)*\\')*$/)
+                or ($tmp2 =~ m/(?<!\\)(\\\\)*\\\[([^\]]|(?<!\\)(\\\\)*\\\[)*$/)) {
                 # Do not change - to \- for
                 #  * \s-n (reduce font size)
                 #  * \(.- (a character named '.-', e.g. '<-')
                 #  * inside a \h'...'
+                #  * inside a \C'...'
+                #  * inside a \[...]
+                #  * inside a \v'...'
                 $tmp = $tmp2."-";
             } else {
                 $tmp = $tmp2."\\-";
@@ -832,6 +891,10 @@ sub post_trans {
         }
         $str = $tmp.$str;
     }
+    }
+
+    # There must not be an end of line inside an inline macro
+    $str =~ s/(E<\.[^>]*)\n([^>]*>)/$1 $2/gs;
 
     # No . or ' on first char, or nroff will think it's a macro
     # * at the beginning of a paragraph, add \& (zero width space) at
@@ -888,10 +951,18 @@ sub post_trans {
 	$str=$done;
     }
 
-    while ($str =~ m/^(.*?)E<([.'][\t ]*.*?)(?<!E<[gl]t)>(.*)$/s) {
+    while ($str =~ m/^(.*?)E<([.'][\t ]*.*?(?<!E<[gl]t))>(.*)$/s) {
         my ($t1, $t2, $t3) = ($1,$2,$3);
         $t1 =~ s/ +$//s;
         $t2 =~ s/\n/ /gs;
+        if ($mdoc_mode) {
+            # restore the punctuaction inside the line (see pre_trans)
+            if ($t3 =~ s/^([.,;:\)\]]+)//s) {
+                my $punctuation = $1;
+                $punctuation =~ s/([.,;:\)\]])/$1 /;
+                $t2 .= " $punctuation";
+            }
+        }
         $t3 =~ s/^ +//s;
         if ($wrap) {
         # The no-wrap case should be checked
@@ -913,8 +984,10 @@ sub post_trans {
     # Don't do that, because we'll go into trouble if previous line was .TP
     # $str =~ s/^\\f([BI])(.*?)\\f[RP]$/\.$1 $2/mg;
     
+    unless ($mdoc_mode) {
     $str =~ s/``/\\\*\(lq/sg;
     $str =~ s/''/\\\*\(rq/sg;
+    }
 
     print STDERR "$str\n" if ($debug{'postrans'});
     return $str;
@@ -1008,7 +1081,7 @@ sub parse{
 
 	if ($line =~ /^[.']/) {
 	    die wrap_mod("po4a::man", dgettext("po4a", "Unparsable line: %s"), $line)
-		unless ($line =~ /^([.']+\\*?)(\\\")(.*)/ ||
+		unless ($line =~ /^([.']+\\*?)(\\["#])(.*)/ ||
 			$line =~ /^([.'])(\S*)(.*)/);
 	    my $arg1=$1;
 	    $arg1 .= $2;
@@ -1022,14 +1095,13 @@ sub parse{
 
 	    # Split on spaces for arguments, but not spaces within double quotes
 	    my @args=();
-	    $line =~ s/\\ /$nbs/g; # This is probably not needed
 	    push @args,$arg1;
-	    if ($macro =~ /^(?:ta|TP)$/) {
+	    if ($macro =~ /^(?:ta|TP|ie|if|de)$/) {
 		# The number of spaces may be critical for the 'ta' macro,
 		# and there is no need to split the arguments.
 		push @args, $arguments;
 	    } else {
-		push @args, splitargs($ref,$arguments)
+		push @args, splitargs($ref,$arguments);
 	    }
 
 
@@ -1041,12 +1113,13 @@ sub parse{
 
 	    # Special case: Don't change these lines
 	    #  .\"  => comments
+	    #  .\#  => comments
 	    #  ."   => comments
 	    #  .    => empty point on the line
 	    #  .tr abcd...
 	    #       => substitution like Perl's tr/ac/bd/ on output.
-	    if ($macro eq '\"' || $macro eq '' || $macro eq 'tr' ||
-                $macro eq '"') {
+	    if ($macro eq '\\"' || $macro eq '' || $macro eq 'tr' ||
+	        $macro eq '"'   || $macro eq '\\#') {
 		$self->pushline($line."\n");
 		goto LINE;
 	    }
@@ -1068,13 +1141,6 @@ sub parse{
 		$wrapped_mode='YES';
 	    }
 	    
-	    # Special case:
-	    #  .Dd => Indicates that this is a mdoc page
-	    if ($macro eq 'Dd') {
-		die wrap_mod("po4a::man", dgettext("po4a",
-		    "This page seems to be a mdoc(7) formatted one. This is not supported (yet)."));
-	    }
-		
 	    unshift @args,$self;
 	    # Apply macro
 	    $self->{type}=$macro;
@@ -1087,12 +1153,12 @@ sub parse{
 		    "Unknown macro '%s'. Remove it from the document, or provide a patch to the po4a team."), $line);
 	    }
 
-	} elsif ($line =~ /^( +)([^. ].*)/) {
+	} elsif ($line =~ /^ +[^. ]/) {
 	    # (Lines containing only spaces are handled as empty lines)
 	    # Not a macro, but not a wrapped paragraph either
 	    $wrapped_mode = $wrapped_mode eq 'YES' ? 'NO' : $wrapped_mode;
 	    $paragraph .= $line."\n";
-	} elsif ($line =~ /^([^.].*)/ && $line !~ /^ *$/) {
+	} elsif ($line =~ /^[^.].*/ && $line !~ /^ *$/) {
 	    # (Lines containing only spaces are handled latter as empty lines)
 	    if ($line =~ /^\\"/) {
 		# special case: the line is entirely a comment, keep the
@@ -1103,7 +1169,10 @@ sub parse{
 		# input line is ignored.
 		$self->pushline($line."\n");
 		goto LINE;
-            } else {
+	    } elsif ($line =~ /^\\#/) {
+		# Special groff comment. Do not keep the new line
+		goto LINE;
+	    } else {
 		# Not a macro
 		# * first, try to handle some "output line continuation" (\c)
 		$paragraph =~ s/\\c *(($FONT_RE)?)\n?$/$1/s;
@@ -1144,10 +1213,36 @@ sub parse{
     }
 } # end of main
 
-
+# Clear all the memories.
+# This should be called between all documents.
+sub re_init {
+    @comments = ();
+    @next_comments = ();
+    set_regular("R");
+    set_font("R");
+    set_font("R");
+    $mdoc_mode = 0;
+}
+# We can't push the header in the first line of the document, as in the
+# other module, because the first line may contain indications on how the
+# man page must be processed.
 sub docheader {
-    return ".\\\" This file was generated with po4a. Translate the source file.\n".
-           ".\\\" \n";
+    re_init();
+    return "";
+}
+
+# The header is pushed just before the .TH macro (this macro is mandatory
+# and must be specified at the begining (there may be macro definitions
+# before).
+sub push_docheader {
+    my $self = shift;
+    $self->pushline(
+".\\\"*******************************************************************\n".
+".\\\"\n".
+".\\\" This file was generated with po4a. Translate the source file.\n".
+".\\\"\n".
+".\\\"*******************************************************************\n"
+    );
 }
 
 # Split request's arguments.
@@ -1206,7 +1301,21 @@ sub splitargs {
                 push @args,$buffer;
                 $buffer = "";
                 $escaped = 0;
+            } else {
+                print STDERR "Quotes: Oops\n"
+                    if ($debug{'splitargs'});
             }
+        } elsif ($elem =~ m/^"(.*?(?<!")(?:"")*)"(?!")(.+)$/) {
+            print STDERR "Quoted: quote without space\n"
+                if ($debug{'splitargs'});
+            my $a = $1;
+            my $b = $2;
+            $a =~ s/""/\\(dq/g;
+            $a =~ s/\Q$nbs/\\ /g;
+            $b =~ s/""/\\(dq/g;
+            $b =~ s/\Q$nbs/\\ /g;
+            push @args,$a;
+            push @args,$b;
         } elsif ($elem =~ m/^"(.*)"$/) {
             print STDERR "Quoted, no space\n"
                 if ($debug{'splitargs'});
@@ -1228,6 +1337,7 @@ sub splitargs {
         } else {
             print STDERR "Unquoted arg, nothing to declare\n"
                 if ($debug{'splitargs'});
+            $elem =~ s/\Q$nbs/\\ /g;
             push @args,$elem;
             $buffer="";
         }
@@ -1296,8 +1406,8 @@ sub splitargs {
 
     sub do_fonts {
         # one argument: a string
-        my $str = $_[0];
-        print STDERR "do_fonts('$str')="
+        my ($str, $ref) = (shift, shift);
+        print STDERR "do_fonts('$str', '$ref')="
             if ($debug{'fonts'});
 
         # restore the font stack
@@ -1329,7 +1439,7 @@ sub splitargs {
                 $elem =~ s/^3/B/;
                 $elem =~ s/^4/(BI/;
 
-            if ($elem =~ /^([1-4]|B|I|R|\(CW|\[\]|\[P\])(.*)$/s) {
+            if ($elem =~ /^([1-4]|B|I|R|\(CW|\[\]|\[P\]|L)(.*)$/s) {
                 # Each element should now start by a recognized font modifier
                 my $new_font = $1;
                 my $arg = $2;
@@ -1346,7 +1456,11 @@ sub splitargs {
                     $str .= "\\f".$elem;
                 }
             } else {
-                die wrap_mod("po4a::man", dgettext("po4a", "Unsupported font in: '%s'."), $elem);
+                die wrap_ref_mod($ref,
+                                 "po4a::man",
+                                 dgettext("po4a",
+                                          "Unsupported font in: '%s'."),
+                                 $elem);
             }
         }
         # Do some simplification (they don't change the font stack)
@@ -1471,6 +1585,11 @@ $macro{'TH'}= sub {
     my ($th,$title,$section,$date,$source,$manual)=@_;
     #Preamble#.TH      title     section   date     source   manual
 #    print STDERR "TH=$th;titre=$title;sec=$section;date=$date;source=$source;manual=$manual\n";
+
+    # Reset the memories
+    re_init();
+    $self->push_docheader();
+
     $self->pushmacro($th,
 		     $self->t($title),
 		     $section,
@@ -1498,7 +1617,12 @@ $macro{'SS'}=$macro{'SH'}=sub {
     }
 };
 
-$macro{'SM'}=\&translate_joined;
+# Macro: .SM [text]
+#     Set the text on the same line or the text on the next line in a
+#     font that is one point size smaller than the default font.
+# FIXME: Maybe we should find a better way to represent this (inline is
+#        not really nice in the PO).
+$inline{'SM'}=1;
 
 # .SP n     Skip n lines (I think)
 $macro{'SP'}=\&untranslated;	
@@ -1641,12 +1765,9 @@ $macro{'UN'}=\&translate_joined;
 $macro{'DT'}=\&noarg;
 $macro{'PD'}=\&untranslated;
 
-#Index. Where's the definition?
-#.IX type content
-$macro{'IX'}=sub {
-    my $self=shift;
-    $self->pushmacro($_[0],$_[1],$self->t($_[2]));
-};
+# Indexing term (printed on standard error).
+# (ms macros)
+$macro{'IX'}=\&translate_each;
 
 ###
 ### groff macros
@@ -1744,7 +1865,7 @@ $macro{'ie'}=$macro{'if'}=sub {
         if ($m eq '.ie') {
             ($line,$ref)=$self->SUPER::shiftline();
             chomp $line;
-            if ($line !~ m/^\.[ \t]*el\s/) {
+            if ($line !~ m/^[.'][ \t]*el(\s|\\\{)/) {
                 die ".ie without .el\n"
             }
             my $paragraph2 = $line;
@@ -1811,8 +1932,10 @@ $macro{'ps'}=\&untranslated;
 # .so filename Include source file.
 # .mso groff variant of .so (other search path)
 $macro{'so'}= $macro{'mso'} = sub {
-    die wrap_mod("po4a::man", dgettext("po4a",
-	"This page includes another file with '%s'. This is not supported yet, but will soon."), $_[1]);
+    warn wrap_mod("po4a::man", dgettext("po4a",
+	"This page includes another file with '%s'. Do not forget to translate this file ('%s')."), $_[1], $_[2]);
+    my $self = shift;
+    $self->pushmacro(@_);
 };
 # .sp     Skip one line vertically.
 # .sp N   Space  vertical distance N
@@ -1883,7 +2006,7 @@ $macro{'TS'}=sub {
 
 ## Builtin register, of course they do not need to be translated
 
-$macro{'F'}=$macro{'H'}=$macro{'V'}=$macro{'A'}=$macro{'P'}=$macro{'T'}=\&untranslated;
+$macro{'F'}=$macro{'H'}=$macro{'V'}=$macro{'A'}=$macro{'T'}=\&untranslated;
 
 ## ms package
 ##
@@ -1963,3 +2086,143 @@ $macro{'UC'}=$macro{'AT'}=\&untranslated;
 #   If the english page needs to specify how a word must be hyphanted, the
 #   translated page may also have this need.
 $macro{'hw'}=\&translate_each;
+
+
+#############################################################################
+#
+# mdoc macros
+#
+# The macros are defined in mdoc(7) and groff_mdoc(7)
+#
+# TBC: Should the font processing be disabled in the mdoc mode?
+#############################################################################
+# FIXME: Maybe we should verify that the page is an mdoc page
+#        (add a flag in Dd, and always check that this flag is set in the
+#        other mdoc macros)
+sub translate_mdoc {
+    my ($self,$macroname,$macroarg)=(shift,shift,join(" ",@_));
+
+    $self->pushline("$macroname ".$self->t($macroarg)."\n");
+}
+#
+# Title Macros
+# ============
+# .Dd   Month day, year                       Document date.
+$macro{'Dd'}=sub {
+    my ($self,$macroname,$macroarg)=(shift,shift,join(" ",@_));
+
+    re_init();
+    $mdoc_mode = 1;
+    $self->push_docheader();
+
+# FIXME: It would be nice if we could switch from one set of macros to the
+# other.
+#
+# This does not work at this time. If we erase the current set of macros,
+# po4a fails when a configuration file uses both mdoc and groff pages.
+#
+#    # Erase the current macro definitions
+#    %macro=();
+#    %inline=();
+#    %no_wrap_begin=();
+#    %no_wrap_end=();
+    # Use the mdoc macros
+    define_mdoc_macros();
+
+    $self->translate_mdoc($macroname,$macroarg);
+};
+
+sub define_mdoc_macros {
+    # .Dt   DOCUMENT_TITLE [section] [volume]     Title, in upper case.
+    $macro{'Dt'}=\&translate_mdoc;
+    # .Os   OPERATING_SYSTEM [version/release]    Operating system (BSD).
+    $macro{'Os'}=\&translate_each;
+    # Keep the quotes e.g. finger.1
+    # Don't add quotes e.g. logger.1
+
+    # Page Layout Macros
+    # ==================
+    # .Sh   Section Headers.
+    # (man mdoc indicates only a limited set of valid headers,
+    # but it should be OK to translate the header)
+    $macro{'Sh'}=\&translate_mdoc;
+    # .Ss   Subsection Headers.
+    $macro{'Ss'}=\&translate_mdoc;
+    # .Pp   Paragraph Break.  Vertical space (one line).
+    $macro{'Pp'}=\&noarg;
+    # .D1   (D-one) Display-one Indent and display one text line.
+    $macro{'D1'}=\&translate_mdoc;
+    # .Dl   (D-ell) Display-one literal.
+    #       Indent and display one line of literal text
+    $macro{'Dl'}=\&translate_mdoc;
+    # .Bd   Begin-display block.
+    # FIXME: Note: there are some options, some of the options argument
+    #        may be translatable (-file <name>, -offset <string>)
+    $no_wrap_begin{'Bd'} = 1;
+    # .Ed   End-display (matches .Bd).
+    $no_wrap_end{'Ed'} = 1;
+    # .Bl   Begin-list.  Create lists or columns.
+    # FIXME: As for .Bd, there are some options
+    $macro{'Bl'}=\&untranslated;
+    # .El   End-list.
+    $macro{'El'}=\&noarg;
+    # .It   List item.
+    # FIXME: Maybe we could extract other modifiers
+    #        as in .It Fl l Ar num
+    $macro{'It'}=\&translate_mdoc;
+
+    # Manual Domain Macros
+    # ====================
+    # FIXME: I think most Manual and General text domain are in the inline category
+    foreach (qw(Ad An Ar Cd Cm Dv Er Ev Fa Fd Fn Ic Li Nm Op Ot Pa St Va Vt Xr)) {
+        $inline{$_} = 1;
+    }
+    # FIXME: some of thes macro introduce a line in bold.
+    #        Using \fP in these line is not supported.
+    #        do_fonts should be called for every inline line
+
+    # General Text Domain
+    # ===================
+    foreach (qw(%A %B %C %D %J %N %O %P %R %T %V
+                Ac Ao Ap Aq At Bc Bf Bo Bq Bx Db Dc Do Dq Ec Ef Em Eo Fx No Ns
+                Pc Pf Po Pq Qc Ql Qo Qq Re Rs Rv Sc So Sq Sm Sx Sy Tn Ux Xc Xo)) {
+        $inline{$_} = 1;
+    }
+
+    # FIXME: Maybe it should be joined with the preceding .Nm
+    $macro{'Nd'}=\&translate_mdoc;
+
+    # Command line flags
+    $inline{'Fl'} = 1;
+    # Exit status
+    $inline{'Ex'} = 1;
+    # Opening option bracket
+    $inline{'Oo'} = 1;
+    # Closing option bracket
+    $inline{'Oc'} = 1;
+    # Begin keep (keep words in the same line)
+    $inline{'Bk'} = 1;
+    # End keep
+    $inline{'Ek'} = 1;
+    # Library Names
+    $inline{'Lb'} = 1;
+    # Function Types
+    $inline{'Ft'} = 1;
+    # Function open (for functions with many arguments)
+    $inline{'Fo'} = 1;
+    # Function close
+    $inline{'Fc'} = 1;
+    # OpenBSD macro
+    $inline{'Ox'} = 1;
+    # BSD/OS Macro
+    $inline{'Bsx'} = 1;
+    # #include statements
+    $macro{'In'} = \&translate_mdoc;
+    # NetBSD Macro
+    $inline{'Nx'} = 1;
+
+    # This macro is a groff macro. I don't know if ot is valid in an mdoc page.
+    # But this is used in some pages and seems to work
+    $macro{'br'}=\&noarg;
+
+} # end of define_mdoc_macros
