@@ -1,5 +1,5 @@
 # Locale::Po4a::Po -- manipulation of po files 
-# $Id: Po.pm,v 1.51 2005/12/04 20:25:52 nekral-guest Exp $
+# $Id: Po.pm,v 1.63 2006/05/05 22:01:44 nekral-guest Exp $
 #
 # This program is free software; you may redistribute it and/or modify it
 # under the terms of GPL (see COPYING).
@@ -56,6 +56,7 @@ This specifies the reference format. It can be one of 'none' to not produce
 any reference, 'noline' to not specify the line number, and 'full' to
 include complete references.
 
+=back
 
 =cut
 
@@ -80,6 +81,7 @@ use vars qw(@ISA @EXPORT);
 
 use Carp qw(croak);
 use File::Path; # mkdir before write
+use File::Copy; # move
 
 use Encode;
 
@@ -89,7 +91,7 @@ my %debug=('canonize'	=> 0,
            'quote'	=> 0,
            'escape'	=> 0,
            'encoding'   => 0,
-           'filter'     => 1);
+           'filter'     => 0);
 
 =head1 Functions about whole message catalogs
 
@@ -133,7 +135,10 @@ sub initialize {
                      $self->{options}{'porefs'});
 
     $self->{po}=();
-    $self->{count}=0;
+    $self->{count}=0;  # number of msgids in the PO
+    # count_doc: number of strings in the document
+    # (duplicate strings counted multiple times)
+    $self->{count_doc}=0;
     $self->{header_comment}=
 	escape_text( " SOME DESCRIPTIVE TITLE\n"
 		    ." Copyright (C) YEAR Free Software Foundation, Inc.\n"
@@ -312,31 +317,44 @@ update a line reference or the POT-Creation-Date field).
 
 =cut
 
+sub move_po_if_needed {
+    my ($new_po, $old_po) = (shift, shift);
+    my $diff;
+
+    if (-e $old_po) {
+        $diff = qx(diff -q -I'^#:' -I'^"POT-Creation-Date:' -I'^"PO-Revision-Date:' $old_po $new_po);
+        if ( $diff eq "" ) {
+            unlink $new_po
+                or die wrap_msg(dgettext("po4a","Can't unlink %s."),
+                                $new_po);
+            # touch the old PO
+            my ($atime, $mtime) = (time,time);
+            utime $atime, $mtime, $old_po;
+        } else {
+            move $new_po, $old_po
+                or die wrap_msg(dgettext("po4a","Can't move %s to %s."),
+                                $new_po, $old_po);
+        }
+    } else {
+        move $new_po, $old_po
+            or die wrap_msg(dgettext("po4a","Can't move %s to %s."),
+                            $new_po, $old_po);
+    }
+}
+
 sub write_if_needed {
     my $self=shift;
     my $filename=shift 
 	or croak (dgettext("po4a","Can't write to a file without filename")."\n");
 
     if (-e $filename) {
-        my ($tmp_filename, $diff);
+        my ($tmp_filename);
         (undef,$tmp_filename)=File::Temp->tempfile($filename."XXXX",
                                                    DIR    => "/tmp",
                                                    OPEN   => 0,
                                                    UNLINK => 0);
         $self->write($tmp_filename);
-        $diff = qx(diff -q -I'#:' -I'POT-Creation-Date:' -I'PO-Revision-Date:' $filename $tmp_filename);
-        if ( $diff eq "" ) {
-            unlink $tmp_filename ||
-                die wrap_msg(dgettext("po4a","Can't unlink %s."),
-                             $tmp_filename);
-            # touch it
-            my ($atime, $mtime) = (time,time);
-            utime $atime, $mtime, $filename;
-        } else {
-            rename $tmp_filename, $filename ||
-                die wrap_msg(dgettext("po4a","Can't rename %s to %s."),
-                             $tmp_filename, $filename);
-        }
+        move_po_if_needed($tmp_filename, $filename);
     } else {
         $self->write($filename);
     }
@@ -367,19 +385,19 @@ sub gettextize {
 
     # Don't fail right now when the entry count does not match. Instead, give it a try so that the user
     # can see where we fail (which is probably where the problem is).
-    if ($poorig->count_entries() > $potrans->count_entries()) {
+    if ($poorig->count_entries_doc() > $potrans->count_entries_doc()) {
 	warn wrap_mod("po4a gettextize", dgettext("po4a",
 	    "Original has more strings than the translation (%d>%d). ".
 	    "Please fix it by editing the translated version to add some dummy entry."),
-		$poorig->count_entries() , $potrans->count_entries());
+		$poorig->count_entries_doc() , $potrans->count_entries_doc());
         $please_fail = 1;      
-    } elsif ($poorig->count_entries() < $potrans->count_entries()) {
+    } elsif ($poorig->count_entries_doc() < $potrans->count_entries_doc()) {
 	warn wrap_mod("po4a gettextize", dgettext("po4a",
 	    "Original has less strings than the translation (%d<%d). ".
 	    "Please fix it by removing the extra entry from the translated file. ".
 	    "You may need an addendum (cf po4a(7)) to reput the chunk in place after gettextization. ".
 	    "A possible cause is that a text duplicated in the original is not translated the same way each time. Remove one of the translations, and you're fine."),
-               $poorig->count_entries(), $potrans->count_entries());
+               $poorig->count_entries_doc(), $potrans->count_entries_doc());
         $please_fail = 1;
     }
 
@@ -400,12 +418,12 @@ sub gettextize {
 	    if $debug{'encoding'};
 
     for (my ($o,$t)=(0,0) ;
-	 $o<$poorig->count_entries() && $t<$potrans->count_entries();
+	 $o<$poorig->count_entries_doc() && $t<$potrans->count_entries_doc();
 	 $o++,$t++) {
 	#
 	# Extract some informations
 	#
-	my ($orig,$trans)=($poorig->msgid($o),$potrans->msgid($t));
+	my ($orig,$trans)=($poorig->msgid_doc($o),$potrans->msgid_doc($t));
 #	print STDERR "Matches [[$orig]]<<$trans>>\n";
 
 	my ($reforig,$reftrans)=($poorig->{po}{$orig}{'reference'},
@@ -443,7 +461,12 @@ sub gettextize {
 	$pores->push_raw('msgid' => $orig, 'msgstr' => $trans, 
 			 'flags' => ($poorig->{po}{$orig}{'flags'} ? $poorig->{po}{$orig}{'flags'} :"")." fuzzy",
 	                 'type'  => $typeorig,
-			 'reference' => $reforig);
+			 'reference' => $reforig,
+			 'conflict' => 1)
+	    unless (defined($pores->{po}{$orig})
+	            and ($pores->{po}{$orig}{'msgstr'} eq $trans))
+	# FIXME: maybe we should be smarter about what reference should be
+	#        sent to push_raw.
     }
     die "$toobad\n" if $please_fail; # make sure we return a useful error message when entry count differ
     return $pores;
@@ -495,7 +518,8 @@ sub filter {
     
     # I dream of a lex in perl :-/
     sub parse_expression {
-	showmethecode("Begin expression");
+	showmethecode("Begin expression")
+	    if $debug{'filter'};
 
 	gloups("Begin of expression expected, got '%s'",$filter[$pos])
 	  unless ($filter[$pos] eq '(');
@@ -503,7 +527,8 @@ sub filter {
 	if ($filter[$pos] eq '&') {
 	    # AND
 	    $pos++;
-	    showmethecode("Begin of AND");
+	    showmethecode("Begin of AND")
+		if $debug{'filter'};
 	    $code .= "(";
 	    while (1) {
 		gloups ("Unfinished AND statement.") 
@@ -569,7 +594,8 @@ sub filter {
 		$pos++;
 		$quoted = 1;
 	    }
-	    showmethecode(($quoted?"Quoted":"Unquoted")." argument of field '$field'");
+	    showmethecode(($quoted?"Quoted":"Unquoted")." argument of field '$field'")
+		if $debug{'filter'};
 
 	    while (!$done) {
 		gloups("Unfinished EQ argument.")
@@ -611,7 +637,8 @@ sub filter {
 	    $code .= "(\$_[$fieldpos] =~ m/$arg/)";
 	    
 	}
-	showmethecode("End of expression");
+	showmethecode("End of expression")
+	    if $debug{'filter'};
         gloups("Unfinished statement.")
 	  if ($pos == $length);
 	gloups("End of expression expected, got '%s'",$filter[$pos])
@@ -624,7 +651,8 @@ sub filter {
     gloups("Garbage at the end of the expression")
       if ($pos != $length);
     $code .= "; }";
-    print STDERR "CODE = $code\n";
+    print STDERR "CODE = $code\n"
+	if $debug{'filter'};
     eval $code;
     die wrap_mod("po4a::po", dgettext("po4a", "Eval failure: %s"), $@)
       if $@;
@@ -653,6 +681,9 @@ sub filter {
 		       'automatic' => $automatic)
 	       if (apply($msgid,$msgstr,$ref,$flags,$comment,$automatic)); # DO NOT CHANGE THE ORDER
     }
+    # delete the apply subroutine
+    # otherwise it will be redefined.
+    undef &apply;
     return $res;
 }
 
@@ -908,6 +939,7 @@ sub push_raw {
 	($entry{'msgid'},$entry{'msgstr'},
 	 $entry{'reference'},$entry{'comment'},$entry{'automatic'},
 	 $entry{'flags'},$entry{'type'});
+    my $keep_conflict = $entry{'conflict'};
 
 #    print STDERR "Push_raw\n";
 #    print STDERR " msgid=>>>$msgid<<<\n" if $msgid;
@@ -944,6 +976,14 @@ sub push_raw {
 		 format_comment(". ",$reference).
 		 quote_text($msgstr));
 
+	    if ($keep_conflict) {
+		$msgstr = "#-#-#-#-#  choice  #-#-#-#-#\\n".
+		          $self->{po}{$msgid}{'msgstr'}."\\n".
+		          "#-#-#-#-#  choice  #-#-#-#-#\\n".
+		          "$msgstr";
+		$msgstr = "#-#-#-#-#  choice  #-#-#-#-#\\n".$msgstr
+		    unless ($msgstr =~ m/^#-#-#-#-#  choice  #-#-#-#-#\\n/s);
+	    } else {
 	    warn wrap_msg(dgettext("po4a",
 	    	"Translations don't match for:\n".
 		"%s\n".
@@ -952,6 +992,7 @@ sub push_raw {
 		" Second translation:\n".
 		"%s\n".
 		" Old translation discarded."),$txt,$first,$second);
+	    }
 	}
     }
     $self->{po}{$msgid}{'reference'} = (defined($self->{po}{$msgid}{'reference'}) ? 
@@ -961,6 +1002,11 @@ sub push_raw {
     $self->{po}{$msgid}{'msgstr'} = $msgstr;
     $self->{po}{$msgid}{'comment'} = $comment;
     $self->{po}{$msgid}{'automatic'} = $automatic;
+    if (defined($self->{po}{$msgid}{'pos_doc'})) {
+        $self->{po}{$msgid}{'pos_doc'} .= " ".$self->{count_doc}++;
+    } else {
+        $self->{po}{$msgid}{'pos_doc'}  = $self->{count_doc}++;
+    }
     unless (defined($self->{po}{$msgid}{'pos'})) {
       $self->{po}{$msgid}{'pos'} = $self->{count}++;
     }
@@ -999,6 +1045,18 @@ sub count_entries($) {
     return $self->{count};
 }
 
+=item count_entries_doc()
+
+Returns the number of entries in document. If a string appears multiple times
+in the document, it will be counted multiple times
+
+=cut
+
+sub count_entries_doc($) {
+    my $self=shift;
+    return $self->{count_doc};
+}
+
 =item msgid($)
 
 Returns the msgid of the given number.
@@ -1015,6 +1073,24 @@ sub msgid($$) {
     return undef;
 }
 
+=item msgid_doc($)
+
+Returns the msgid with the given position in the document.
+
+=cut
+
+sub msgid_doc($$) {
+    my $self=shift;
+    my $num=shift;
+    
+    foreach my $msgid ( keys %{$self->{po}} ) {
+        foreach my $pos (split / /, $self->{po}{$msgid}{'pos_doc'}) {
+            return $msgid if ($pos eq $num);
+        }
+    }
+    return undef;
+}
+
 =item get_charset()
 
 Returns the character set specified in the po header. If it hasn't been
@@ -1074,7 +1150,7 @@ sub unescape_text {
               /$1\n/sgx;   # single string, match globally, allow comments
     # unescape tabulations
     $text =~ s/(          # $1:
-                (^|[^\\]) #    beginning of the line or any char
+                (\G|[^\\])#    beginning of the line or any char
                           #    different from '\'
                 (\\\\)*   #    followed by any even number of '\'
                )\\t       # and followed by an escaped tabulation
@@ -1199,7 +1275,7 @@ sub wrap {
 	$res.="$line\n";
     }
     # Restore the original trailing spaces
-    chomp $res;
+    $res =~ s/\s+$//s;
     if ($text =~ m/(\s+)$/s) {
 	$res .= $1;
     }
