@@ -295,7 +295,23 @@ sub translate {
         return $string.($options{'wrap'}?"\n":"");
     }
 
-    return $self->SUPER::translate($string,$ref,$type,%options);
+    $string = $self->SUPER::translate($string,$ref,$type,%options);
+
+    $string = $self->post_trans($string,$ref,$type);
+
+    return $string;
+}
+
+sub post_trans {
+    my ($self,$str,$ref,$type)=@_;
+
+    # Change ascii non-breaking space to an &nbsp;
+    my $nbs_out = "\xA0";
+    my $enc_length = Encode::from_to($nbs_out, "latin1",
+                                               $self->get_out_charset);
+    $str =~ s/\Q$nbs_out/&nbsp;/g if defined $enc_length;
+
+    return $str;
 }
 
 #
@@ -339,7 +355,7 @@ sub parse_file {
     while (<IN>) {
 	$origfile .= $_;
     }
-    close IN || die wrap_mod("po4a::sgml", dgettext("po4a", "Can't close %s: %s"), $mastername, $!);
+    close IN or die wrap_mod("po4a::sgml", dgettext("po4a", "Can't close %s: %s"), $mastername, $!);
     # Detect the XML pre-prolog
     if ($origfile =~ s/^(\s*<\?xml[^?]*\?>)//) {
 	warn wrap_mod("po4a::sgml", dgettext("po4a",
@@ -414,7 +430,7 @@ sub parse_file {
     } elsif ($prolog =~ /docbook/i) {
 	$self->set_tags_kind("translate" => "abbrev acronym arg artheader attribution ".
 	                                    "date ".
-	                                    "editor entry ".
+	                                    "edition editor entry ".
 	                                    "figure ".
 	                                    "glosssee glossseealso glossterm ".
 	                                    "holder ".
@@ -429,7 +445,7 @@ sub parse_file {
 	                     "indent"    => "abstract answer appendix article articleinfo audioobject author authorgroup ".
 	                                    "bibliodiv bibliography blockquote blockinfo book bookinfo bridgehead ".
 	                                    "callout calloutlist caption caution chapter cmdsynopsis copyright ".
-	                                    "dedication ".
+	                                    "dedication docinfo ".
 	                                    "entry ".
 	                                    "formalpara ".
 	                                    "glossary glossdef glossdiv glossentry glosslist group ".
@@ -441,7 +457,7 @@ sub parse_file {
 	                                    "objectinfo orderedlist ".
 	                                    "part partintro preface procedure publisher ".
 	                                    "qandadiv qandaentry qandaset question ".
-	                                    "reference refsect1 refentry refentryinfo refmeta refnamediv refsect1 refsect1info refsect2 refsect2info refsect3 refsect3info refsection refsectioninfo refsynopsisdiv refsynopsisdivinfo revision revdescription row ".
+	                                    "reference refentry refentryinfo refmeta refnamediv refsect1 refsect1info refsect2 refsect2info refsect3 refsect3info refsection refsectioninfo refsynopsisdiv refsynopsisdivinfo revision revdescription row ".
 	                                    "screenshot sect1 sect1info sect2 sect2info sect3 sect3info sect4 sect4info sect5 sect5info section sectioninfo seglistitem segmentedlist set setindex setinfo shortcut simplelist simplemsgentry simplesect step synopfragment ".
 	                                    "table tbody textobject tgroup thead tip toc ".
 	                                    "variablelist varlistentry videoobject ".
@@ -563,7 +579,7 @@ sub parse_file {
 	        # find the file.
 	        $origfile =~ s/(<!ENTITY\s*%\s*\Q$key\E\s+SYSTEM\s*")\Q$origfilename\E("\s*>)/$1$filename$2/gsi;
 	    }
-	    if (defined $ignored_inclusion{$key} || !-e $filename) {
+	    if (defined $ignored_inclusion{$key} or !-e $filename) {
 		# We won't expand this entity.
 		# And we avoid nsgmls to do so.
 		$prolog = "$begin<!--{PO4A-ent-beg-$key}$filename".
@@ -726,7 +742,7 @@ sub parse_file {
 	    print STDERR "substitute $2\n" if ($debug{'entities'});
         }
     }
-    $origfile=~s/\G(.*?)&([A-Za-z_:][-_:.A-Za-z0-9]*|#[0-9]+|#x[0-9a-fA-F]+);/$1\{PO4A-amp\}$2;/gs;
+    $origfile=~s/\G(.*?)&([A-Za-z_:][-_:.A-Za-z0-9]*|#[0-9]+|#x[0-9a-fA-F]+)\b/$1\{PO4A-amp\}$2/gs;
 
     if ($debug{'refs'}) {
 	print "XX Resulting shifts\n";
@@ -740,7 +756,7 @@ sub parse_file {
                                               DIR    => "/tmp",
                                               UNLINK => 0);
     print $tmpfh $origfile;
-    close $tmpfh || die wrap_mod("po4a::sgml", dgettext("po4a", "Can't close tempfile: %s"), $!);
+    close $tmpfh or die wrap_mod("po4a::sgml", dgettext("po4a", "Can't close tempfile: %s"), $!);
 
     my $cmd="nsgmls -l -E 0 -wno-valid < $tmpfile".
             ($debug{'nsgmls'}?"":" 2>/dev/null")." |";
