@@ -71,8 +71,9 @@ require Exporter;
 use vars qw(@ISA @EXPORT);
 @ISA = qw(Locale::Po4a::TransTractor);
 @EXPORT = qw(%commands %environments
-             $RE_ESCAPE $ESCAPE
+             $RE_ESCAPE $ESCAPE $RE_VERBATIM
              $no_wrap_environments
+             $verbatim_environments
              %separated_command
              %separated_environment
              &generic_command
@@ -100,9 +101,16 @@ our %env_separators =();
 # The escape character used to introduce commands.
 our $RE_ESCAPE = "\\\\";
 our $ESCAPE    = "\\";
+# match the beginning of a verbatim block
+our $RE_VERBATIM = "\\\\begin\\{(?:verbatim)\\*?\\}";
+# match the beginning of a comment.
+# NOTE: It must contain a group, with chars preceding the comment
+our $RE_PRE_COMMENT= "(?<!\\\\)(?:\\\\\\\\)*";
+our $RE_COMMENT= "\\\%";
 
 # Space separated list of environments that should not be re-wrapped.
 our $no_wrap_environments = "verbatim";
+our $verbatim_environments = "verbatim";
 # hash with the commands that have to be separated (or have to be joined).
 # 3 modes are currently used:
 #  '*' The command is separated if it appear at an extremity of a
@@ -138,6 +146,10 @@ B<INLINE CUSTOMIZATION> section.
 You can use this option if it is not possible to put the definitions in
 the document being translated.
 
+=item verbatim
+
+Coma-separated list of environments which should be taken as verbatim.
+
 =back
 
 Using these options permits to override the behaviour of the commands defined
@@ -263,7 +275,7 @@ my $my_dirname;
 # See read_file.
 our @exclude_include;
 
-my %type_end=('{'=>'}', '['=>']');
+my %type_end=('{'=>'}', '['=>']', ' '=>'');
 
 #########################
 #### DEBUGGING STUFF ####
@@ -574,17 +586,9 @@ sub get_trailing_command {
                              $command, $reason);
         }
         if (@$remainder) {
-            # FIXME: we should also keep the spaces to be idempotent
-            my ($temp,$type,$arg);
-            while (@$remainder) {
-                $type = shift @$remainder;
-                $arg  = shift @$remainder;
-                $temp .= $type.$arg.$type_end{$type};
-                # And remove the same number of arguments from @args
-                pop @args;
-                pop @args;
-            }
-            $buffer .= $temp;
+            # There are some arguments after the command.
+            # We can't extract this comand.
+            $command = "";
         }
     }
 
@@ -625,8 +629,8 @@ sub translate_buffer {
     }
     # verbatim blocks.
     # Buffers starting by \end{verbatim} are handled after.
-    if (@env and $env[-1] eq "verbatim" and $buffer !~ m/^\n?\\end\{verbatim\*?\}/) {
-        if($buffer =~ m/^(.*?)(\n?\\end\{verbatim\*?\}.*)$/s) {
+    if (in_verbatim(@env) and $buffer !~ m/^\n?\Q$ESCAPE\Eend\{$env[-1]\*?\}/) {
+        if($buffer =~ m/^(.*?)(\n?\Q$ESCAPE\Eend\{$env[-1]\*?\}.*)$/s) {
             # end of a verbatim block
             my ($begin, $end) = ($1?$1:"", $2);
             my ($t1, $t2) = ("", "");
@@ -649,7 +653,7 @@ sub translate_buffer {
         }
     }
     # early detection of verbatim environment
-    if ($buffer =~ /^(\\begin\{verbatim\*?\}\n?)(.*)$/s and length $2) {
+    if ($buffer =~ /^($RE_VERBATIM\n?)(.*)$/s and length $2) {
         my ($begin, $end) = ($1, $2);
         my ($t1, $t2) = ("", "");
         ($t1, @env) = translate_buffer($self, $begin, @env);
@@ -681,10 +685,10 @@ sub translate_buffer {
 
     # remove comments from the buffer.
     # Comments are stored in an array and shown as comments in the PO.
-    while ($buffer =~ m/((?<!\\)(?:\\\\)*)%([^\n]*)(\n[ \t]*)(.*)$/s) {
+    while ($buffer =~ m/($RE_PRE_COMMENT)$RE_COMMENT([^\n]*)(\n[ \t]*)(.*)$/s) {
         my $comment = $2;
         my $end = "";
-        if ($4 =~ m/^\n/s and $buffer !~ m/^%/s) {
+        if ($4 =~ m/^\n/s and $buffer !~ m/^$RE_COMMENT/s) {
             # a line with comments, followed by an empty line.
             # Keep the empty line, but remove the comment.
             # This is an empirical heuristic, but seems to work;)
@@ -693,7 +697,7 @@ sub translate_buffer {
         if (defined $comment and $comment !~ /^\s*$/s) {
             push @comments, $comment;
         }
-        $buffer =~ s/((?<!\\)(?:\\\\)*)%([^\n]*)(\n[ \t]*)/$1$end/s;
+        $buffer =~ s/($RE_PRE_COMMENT)$RE_COMMENT([^\n]*)(\n[ \t]*)/$1$end/s;
     }
     
 
@@ -1053,7 +1057,9 @@ sub parse_definition_line {
     } elsif ($line =~ /^separator\s+(\w+(?:\[#[0-9]+\]))\s+\"(.*)\"\s*$/) {
         my $env = $1; # This is not necessarily an environment.
                       # It can also be smth like 'title{#1}'.
-        $env_separators{$env} = $2
+        $env_separators{$env} = $2;
+    } elsif ($line =~ /^verbatim\s+environment\s+(\w+)\s+$/) {
+        register_verbatim_environment($1);
     }
 }
 
@@ -1069,14 +1075,14 @@ sub is_closed {
     my $opening = 0;
     # FIXME: { and } should not be counted in verbatim blocks
     # Remove comments
-    $tmp =~ s/(?<!\\)(?:\\\\)*%.*//mg;
+    $tmp =~ s/($RE_PRE_COMMENT)$RE_COMMENT.*//mg;
     while ($tmp =~ /^.*?(?<!\\)(?:\\\\)*\{(.*)$/s) {
         $opening += 1;
         $tmp = $1;
     }
     $tmp = $paragraph;
     # Remove comments
-    $tmp =~ s/(?<!\\)(?:\\\\)*%.*//mg;
+    $tmp =~ s/($RE_PRE_COMMENT)$RE_COMMENT.*//mg;
     while ($tmp =~ /^.*?(?<!\\)(?:\\\\)*\}(.*)$/s) {
         $closing += 1;
         $tmp = $1;
@@ -1084,6 +1090,18 @@ sub is_closed {
     return $opening eq $closing;
 }
 
+sub in_verbatim {
+    foreach my $e1 (@_) {
+        foreach my $e2 (split(' ', $verbatim_environments)) {
+            if ($e1 eq $e2) {
+                return 1;
+            }
+        }
+    }
+
+    return 0;
+}
+
 #############################
 #### MAIN PARSE FUNCTION ####
 #############################
@@ -1117,10 +1135,9 @@ sub parse {
 #       middle of a line. (This is only an issue if the verbatim
 #       environment contains an un-closed bracket)
         if (   ($closed and ($line =~ /^\s*$/ or
-                             $line =~ /^\s*\\begin{verbatim}\s*$/))
-            or (    defined $env[-1]
-                and $env[-1] eq "verbatim"
-                and $line =~ /^\s*\\end{verbatim}\s*$/)) {
+                             $line =~ /^\s*$RE_VERBATIM\s*$/))
+            or (in_verbatim(@env) and $line =~ /^\s*\Q$ESCAPE\Eend{$env[-1]}\s*$/)
+           ) {
             # An empty line. This indicates the end of the current
             # paragraph.
             $paragraph .= $line."\n";
@@ -1147,7 +1164,6 @@ sub parse {
     }
 } # end of parse
 
-
 =item docheader
 
 =back
@@ -1308,12 +1324,13 @@ TEST_TYPE:
                 $count++;
                 goto TEST_TYPE;
             } else {
+                my $reason = dgettext("po4a",
+                                      "An optional argument ".
+                                      "was provided, but a mandatory one ".
+                                      "is expected.");
                 die wrap_ref_mod($self->{ref}, "po4a::tex",
-                                 dgettext("po4a",
-                                     "Command '%s': An optional argument ".
-                                     "was provided, but a mandatory one ".
-                                     "is expected.")."\n",
-                                 $command);
+                                 dgettext("po4a", "Command '%s': %s")."\n",
+                                 $command, $reason);
             }
             if ($have_to_be_translated) {
                 ($t, @e) = translate_buffer($self,$opt,(@$env,$command.$type."#".$count.$type_end{$type}));
@@ -1343,7 +1360,7 @@ TEST_TYPE:
 }
 
 sub register_generic_command {
-    if ($_[0] =~ m/^(.*),((\{_?\}|\[_?\])*)$/) {
+    if ($_[0] =~ m/^(.*),((\{_?\}|\[_?\]| _? )*)$/) {
         my $command = $1;
         my $arg_types = $2;
         if ($command =~ /^([-*+])(.*)$/) {
@@ -1354,7 +1371,7 @@ sub register_generic_command {
         my @translated = ();
         while (    defined $arg_types
                and length $arg_types
-               and $arg_types =~ m/^(?:([\{\[])(_?)[\}\]])(.*)$/) {
+               and $arg_types =~ m/^(?:([\{\[ ])(_?)[\}\] ])(.*)$/) {
             push @types, $1;
             push @translated, ($2 eq "_")?1:0;
             $arg_types = $3;
@@ -1419,12 +1436,13 @@ TEST_TYPE:
                 $count++;
                 goto TEST_TYPE;
             } else {
+                my $reason = dgettext("po4a",
+                                      "An optional argument ".
+                                      "was provided, but a mandatory one ".
+                                      "is expected.");
                 die wrap_ref_mod($self->{ref}, "po4a::tex",
-                                 dgettext("po4a",
-                                     "Command '%s': An optional argument ".
-                                     "was provided, but a mandatory one ".
-                                     "is expected.")."\n",
-                                 $command);
+                                 dgettext("po4a", "Command '%s': %s")."\n",
+                                 $command, $reason);
             }
 
             if ($have_to_be_translated) {
@@ -1507,7 +1525,7 @@ TEST_TYPE:
         } else {
             $check = 0;
             $reason = dgettext("po4a",
-                               "Command '%s': An optional argument was ".
+                               "An optional argument was ".
                                "provided, but a mandatory one is expected.");
         }
     }
@@ -1542,6 +1560,15 @@ sub register_generic_environment {
     }
 }
 
+sub register_verbatim_environment {
+    my $env = shift;
+    $no_wrap_environments .= " $env";
+    $verbatim_environments .= " $env";
+    $RE_VERBATIM = "\\\\begin\\{(?:".
+                   join("|", split(/ /, $verbatim_environments)).
+                   ")\\*?\\}";
+}
+
 ####################################
 ### INITIALIZATION OF THE PARSER ###
 ####################################
@@ -1552,6 +1579,7 @@ sub initialize {
     $self->{options}{'definitions'}='';
     $self->{options}{'exclude_include'}='';
     $self->{options}{'no_wrap'}='';
+    $self->{options}{'verbatim'}='';
     $self->{options}{'debug'}='';
     $self->{options}{'verbose'}='';
 
@@ -1586,6 +1614,12 @@ sub initialize {
         }
     }
 
+    if ($options{'verbatim'}) {
+        foreach (split(/,/, $options{'verbatim'})) {
+            register_verbatim_environment($_);
+        }
+    }
+
     if ($options{'definitions'}) {
         $self->parse_definition_file($options{'definitions'})
     }
@@ -1601,12 +1635,6 @@ It was tested on a book and with the Python documentation.
 
 =over 4
 
-=item other categories
-
-A verbatim category may be needed to indicate that po4a should not attempt
-to rewrap lines, that percent signs do not introduce any comment, and that
-a brackets may not be closed.
-
 =item Automatic detection of new commands
 
 The TeX module could parse the newcommand arguments and try to guess the
@@ -1618,6 +1646,12 @@ translated.
 When \item is used as an environment separator, the item argument is
 attached to the following string.
 
+=item Some commands should be added to the environment stack
+
+These commands should be specified by couples.
+This could allow to specify commands beginning or ending a verbatim
+environment.
+
 =item Others
 
 Various other points are tagged TODO in the source.
